# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'camera_control_gui.ui',
# licensing of 'camera_control_gui.ui' applies.
#
# Created: Wed Sep 12 17:51:00 2018
#      by: pyside2-uic  running on PySide2 5.11.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

from PyQt5.QtWidgets import QApplication as QtCompat


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(505, 584)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_3 = QtWidgets.QLabel(self.centralwidget)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_2.addWidget(self.label_3)
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.lineEditServerHostName = QtWidgets.QLineEdit(self.centralwidget)
        self.lineEditServerHostName.setObjectName("lineEditServerHostName")
        self.horizontalLayout_2.addWidget(self.lineEditServerHostName)
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_2.addWidget(self.label_2)
        self.lineEditServerPort = QtWidgets.QLineEdit(self.centralwidget)
        self.lineEditServerPort.setMinimumSize(QtCore.QSize(61, 0))
        self.lineEditServerPort.setObjectName("lineEditServerPort")
        self.horizontalLayout_2.addWidget(self.lineEditServerPort)
        self.pushButtonConnectToServer = QtWidgets.QPushButton(self.centralwidget)
        self.pushButtonConnectToServer.setObjectName("pushButtonConnectToServer")
        self.horizontalLayout_2.addWidget(self.pushButtonConnectToServer)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.imageShowWidget = ImageShowBasicWidget(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.imageShowWidget.sizePolicy().hasHeightForWidth())
        self.imageShowWidget.setSizePolicy(sizePolicy)
        self.imageShowWidget.setObjectName("imageShowWidget")
        self.verticalLayout.addWidget(self.imageShowWidget)
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.labelFrameCoordsUnderMouse = QtWidgets.QLabel(self.centralwidget)
        self.labelFrameCoordsUnderMouse.setObjectName("labelFrameCoordsUnderMouse")
        self.gridLayout_2.addWidget(self.labelFrameCoordsUnderMouse, 0, 0, 1, 1)
        self.labelFrameValueUnderMouse = QtWidgets.QLabel(self.centralwidget)
        self.labelFrameValueUnderMouse.setObjectName("labelFrameValueUnderMouse")
        self.gridLayout_2.addWidget(self.labelFrameValueUnderMouse, 0, 1, 1, 1)
        self.labelRefreshRate = QtWidgets.QLabel(self.centralwidget)
        self.labelRefreshRate.setObjectName("labelRefreshRate")
        self.gridLayout_2.addWidget(self.labelRefreshRate, 0, 2, 1, 1)
        self.labelFrameValue5percentile = QtWidgets.QLabel(self.centralwidget)
        self.labelFrameValue5percentile.setObjectName("labelFrameValue5percentile")
        self.gridLayout_2.addWidget(self.labelFrameValue5percentile, 1, 0, 1, 1)
        self.labelFrameValue95percentile = QtWidgets.QLabel(self.centralwidget)
        self.labelFrameValue95percentile.setObjectName("labelFrameValue95percentile")
        self.gridLayout_2.addWidget(self.labelFrameValue95percentile, 1, 1, 1, 1)
        self.labelFrameCounter = QtWidgets.QLabel(self.centralwidget)
        self.labelFrameCounter.setObjectName("labelFrameCounter")
        self.gridLayout_2.addWidget(self.labelFrameCounter, 1, 2, 1, 1)
        self.labelFrameValueMin = QtWidgets.QLabel(self.centralwidget)
        self.labelFrameValueMin.setObjectName("labelFrameValueMin")
        self.gridLayout_2.addWidget(self.labelFrameValueMin, 2, 0, 1, 1)
        self.labelFrameValueMax = QtWidgets.QLabel(self.centralwidget)
        self.labelFrameValueMax.setObjectName("labelFrameValueMax")
        self.gridLayout_2.addWidget(self.labelFrameValueMax, 2, 1, 1, 1)
        self.labelFrameShape = QtWidgets.QLabel(self.centralwidget)
        self.labelFrameShape.setObjectName("labelFrameShape")
        self.gridLayout_2.addWidget(self.labelFrameShape, 2, 2, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout_2)
        self.groupBox = QtWidgets.QGroupBox(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setMinimumSize(QtCore.QSize(481, 134))
        self.groupBox.setObjectName("groupBox")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.comboBoxRequestedBinning = QtWidgets.QComboBox(self.groupBox)
        self.comboBoxRequestedBinning.setObjectName("comboBoxRequestedBinning")
        self.gridLayout.addWidget(self.comboBoxRequestedBinning, 2, 3, 1, 1)
        self.lineEditCurrentBinning = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditCurrentBinning.setObjectName("lineEditCurrentBinning")
        self.gridLayout.addWidget(self.lineEditCurrentBinning, 2, 1, 1, 2)
        self.labelExposureTimeInMs = QtWidgets.QLabel(self.groupBox)
        self.labelExposureTimeInMs.setObjectName("labelExposureTimeInMs")
        self.gridLayout.addWidget(self.labelExposureTimeInMs, 0, 0, 1, 2)
        self.labelFramerateInHz = QtWidgets.QLabel(self.groupBox)
        self.labelFramerateInHz.setObjectName("labelFramerateInHz")
        self.gridLayout.addWidget(self.labelFramerateInHz, 1, 0, 1, 1)
        self.lineEditCurrentExposureTime = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditCurrentExposureTime.setObjectName("lineEditCurrentExposureTime")
        self.gridLayout.addWidget(self.lineEditCurrentExposureTime, 0, 2, 1, 1)
        self.lineEditCurrentFramerate = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditCurrentFramerate.setObjectName("lineEditCurrentFramerate")
        self.gridLayout.addWidget(self.lineEditCurrentFramerate, 1, 1, 1, 2)
        self.labelBinning = QtWidgets.QLabel(self.groupBox)
        self.labelBinning.setObjectName("labelBinning")
        self.gridLayout.addWidget(self.labelBinning, 2, 0, 1, 1)
        self.lineEditRequestedExposureTime = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditRequestedExposureTime.sizePolicy().hasHeightForWidth())
        self.lineEditRequestedExposureTime.setSizePolicy(sizePolicy)
        self.lineEditRequestedExposureTime.setObjectName("lineEditRequestedExposureTime")
        self.gridLayout.addWidget(self.lineEditRequestedExposureTime, 0, 3, 1, 1)
        self.lineEditRequestedFramerate = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditRequestedFramerate.setObjectName("lineEditRequestedFramerate")
        self.gridLayout.addWidget(self.lineEditRequestedFramerate, 1, 3, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 1, 4, 1, 1)
        self.horizontalLayout.addLayout(self.gridLayout)
        self.verticalLayout.addWidget(self.groupBox)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar()
        self.menubar.setGeometry(QtCore.QRect(0, 0, 505, 22))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtCompat.translate("MainWindow", "MainWindow", None, -1))
        self.label_3.setText(QtCompat.translate("MainWindow", "Server", None, -1))
        self.label.setText(QtCompat.translate("MainWindow", "Hostname", None, -1))
        self.lineEditServerHostName.setText(QtCompat.translate("MainWindow", "192.168.1.1", None, -1))
        self.label_2.setText(QtCompat.translate("MainWindow", "Port", None, -1))
        self.lineEditServerPort.setText(QtCompat.translate("MainWindow", "7100", None, -1))
        self.pushButtonConnectToServer.setText(QtCompat.translate("MainWindow", "Connect", None, -1))
        self.labelFrameCoordsUnderMouse.setText(QtCompat.translate("MainWindow", "x,y: 1000,1000", None, -1))
        self.labelFrameValueUnderMouse.setText(QtCompat.translate("MainWindow", "value: 99999", None, -1))
        self.labelRefreshRate.setText(QtCompat.translate("MainWindow", "Refresh rate: 1000 Hz", None, -1))
        self.labelFrameValue5percentile.setText(QtCompat.translate("MainWindow", "5%: 100000", None, -1))
        self.labelFrameValue95percentile.setText(QtCompat.translate("MainWindow", "95%: 100000", None, -1))
        self.labelFrameCounter.setText(QtCompat.translate("MainWindow", "Frame counter: 100000000", None, -1))
        self.labelFrameValueMin.setText(QtCompat.translate("MainWindow", "Min: 100000", None, -1))
        self.labelFrameValueMax.setText(QtCompat.translate("MainWindow", "Max: 100000", None, -1))
        self.labelFrameShape.setText(QtCompat.translate("MainWindow", "Frame size: 1000 x 1000", None, -1))
        self.groupBox.setTitle(QtCompat.translate("MainWindow", "Controls", None, -1))
        self.labelExposureTimeInMs.setText(QtCompat.translate("MainWindow", "Exposure Time [ms]", None, -1))
        self.labelFramerateInHz.setText(QtCompat.translate("MainWindow", "Framerate [Hz]", None, -1))
        self.labelBinning.setText(QtCompat.translate("MainWindow", "Binning", None, -1))


from pysilico.gui.image_show_widget.image_show_basic_widget import ImageShowBasicWidget
